use axum::{
    http::{StatusCode, header},
    response::{IntoResponse, Response},
};

use super::TAG;

// Source archive, generated at build time.
const SOURCE_ARCHIVE: &[u8] = include_bytes!(concat!(env!("OUT_DIR"), "/archive.zip"));

// App name from Cargo.toml
const APP_NAME: &str = env!("CARGO_PKG_NAME");

// Version from Cargo.toml
const APP_VERSION: &str = env!("CARGO_PKG_VERSION");

/// Serve the source code archive as a downloadable zip file.
#[utoipa::path(
    get,
    path = "/code",
    tag = TAG,
    responses(
        (status = 200,
         description = "Source code archive as zip file",
         content_type = "application/zip",
         headers(
             ("content-disposition" = String, description = "Attachment filename"),
             ("content-length" = String, description = "File size in bytes")
         )
        ),
    )
)]
pub async fn route() -> Result<Response, StatusCode> {
    let filename = format!("{}-{}-source.zip", APP_NAME, APP_VERSION);
    let content_length = SOURCE_ARCHIVE.len().to_string();
    let content_disposition = format!("attachment; filename=\"{}\"", filename);

    let headers = [
        (header::CONTENT_TYPE, "application/zip"),
        (header::CONTENT_DISPOSITION, content_disposition.as_str()),
        (header::CONTENT_LENGTH, content_length.as_str()),
    ];

    Ok((headers, SOURCE_ARCHIVE).into_response())
}
