use lopdf::Document;
use snafu::{ResultExt, Snafu};

use super::extractor::{Extractor, PdfSnafu};

/// PDF text extractor.
pub struct Pdf;

/// Extract text from all pages of a PDF document.
fn get_pdf_text(doc: &Document) -> Result<String, lopdf::Error> {
    let pages: Vec<u32> = doc.get_pages().keys().cloned().collect();
    let text = doc.extract_text(&pages)?;

    Ok(text)
}

/// Errors that occur during PDF text extraction.
#[derive(Debug, Snafu)]
pub enum PdfExtractionError {
    #[snafu(display("Failed to open pdf."))]
    OpenPdf { source: lopdf::Error },
    #[snafu(display("Failed to extract content from pdf."))]
    ReadPdf { source: lopdf::Error },
}

impl Extractor for Pdf {
    fn extract(input: &[u8]) -> Result<String, super::extractor::ExtractionError> {
        let doc = Document::load_from(input)
            .context(OpenPdfSnafu)
            .context(PdfSnafu)?;
        get_pdf_text(&doc).context(ReadPdfSnafu).context(PdfSnafu)
    }
}
